==> apeman-api-fs-1.2.0.tgz: package/lib/define.js <==
/**
 * Define api of apeman-api-fs
 * @function define
 * @param {Object} [options] - Optional settings
 * @param {boolean} [options.logging] - Enable logger
 * @param {string} [options.root=process.cwd()] - Root directory path
 * @returns {Object} - API configuration
 */
'use strict'

const co = require('co')
const { mkdirpAsync, existsAsync, statAsync } = require('asfs')
const defaults = require('defaults')
const filemode = require('filemode')
const filelink = require('filelink')
const aglob = require('aglob')
const aslogger = require('aslogger')
const filecopy = require('filecopy')
const apemanApiPath = require('apeman-api-path')
const describe = require('./describe')

let modeString = (mode) => mode.toString(8)

/** @lends define */
function define (options = {}) {
  let logger = aslogger({ disabled: !options.logging })
  let { root } = defaults(options, {
    root: process.cwd()
  })

  let path = apemanApiPath({ root, logging: false })

  return {
    ping (pong = 'pong') {
      return co(function * () {
        logger.trace('pong:', pong)
        return pong
      })
    },
    glob (pattern, options) {
      const s = this
      let args = [ ...arguments ]
      return co(function * () {
        let filenames = yield aglob(...args)
        for (let i = 0; i < filenames.length; i++) {
          let filename = filenames[ i ]
          filenames[ i ] = yield path.resolve(filename)
        }
        return filenames
      })
    },
    exists (filename) {
      const s = this
      return co(function * () {
        let resolved = yield path.resolve(filename)
        return yield existsAsync(resolved)
      })
    },
    stat (filename) {
      const s = this
      return co(function * () {
        let resolved = yield path.resolve(filename)
        return yield statAsync(resolved)
      })
    },
    mkdir (dirname) {
      const s = this
      return co(function * () {
        let result = yield s.mkdirAll([ dirname ])
        return !!result[ 0 ]
      })
    },
    mkdirAll (dirnames) {
      const s = this
      return co(function * () {
        let results = []
        for (let dirname of dirnames) {
          let resolved = yield path.resolve(dirname)
          let exists = yield s.exists(resolved)
          if (exists) {
            continue
          }
          yield mkdirpAsync(resolved)
          logger.debug(`Directory generated: ${dirname}`)
          results.push(dirname)
        }
        return results
      })
    },
    chmod (pattern, mode) {
      const s = this
      return s.chmodAll({ [pattern]: mode })
    },
    chmodAll (permissions) {
      const s = this
      return co(function * () {
        let results = []
        for (let pattern of Object.keys(permissions)) {
          let mode = permissions[ pattern ]
          let filenames = yield s.glob(pattern)
          for (let filename of filenames) {
            let changed = yield filemode(filename, mode, {})
            Object.keys(changed).forEach((filename) => {
              let { from, to } = changed[ filename ]
              logger.debug(`Permission changed: ${filename} ${modeString(from)} to ${modeString(to)}`)
              results.push(filename)
            })
          }
        }
        return results
      })
    },
    copy (src, dest) {
      const s = this
      return s.copyAll({ [src]: dest })
    },
    copyAll (config) {
      const s = this
      return co(function * () {
        let results = []
        for (let src of Object.keys(config)) {
          let dest = config[ src ]
          src = yield path.resolve(src)
          dest = yield path.resolve(dest)
          let copied = yield filecopy(src, dest, {
            mkdirp: true
          })
          let changed = Object.keys(copied).shift()
          if (changed) {
            logger.debug(`File copied: ${src} -> ${dest}`)
            results.push(changed)
          }
        }
        return results
      })
    },
    symlink (src, dest) {
      const s = this
      return s.symlinkAll({ [src]: dest })
    },
    symlinkAll (config) {
      const s = this
      return co(function * () {
        let results = []
        for (let src of Object.keys(config)) {
          let dest = config[ src ]
          src = yield path.resolve(src)
          dest = yield path.resolve(dest)
          let changed = yield filelink(src, dest, {
            type: 'symlink',
            mkdirp: true,
            force: true
          })
          if (changed) {
            logger.debug(`File linked: ${src} -> ${dest}`)
            results.push({ src, dest })
          }
        }
        return results
      })
    },
    /**
     * A hook which runs before method invoked
     * @param {string} methodName - Name of method to invoke
     * @param {Array} params - Prams for invocation
     * @returns {Promise}
     */
    $before (methodName, params) {
      const s = this
      let { state } = s
      return co(function * () {
        state.invokedAt = new Date()
        logger.info(`${methodName} started...`)
      })
    },
    /**
     * A hook which runs after method invoked
     * @param {string} methodName - Name of method to invoke
     * @param {Array} params - Prams for invocation
     * @param {?*} returns - Return values
     * @returns {Promise}
     */
    $after (methodName, params, returns) {
      const s = this
      let { state } = s
      return co(function * () {
        let took = new Date() - state.invokedAt
        logger.info(`...${methodName} done! (${took}ms)`)
      })
    },
    /**
     * Method specifications
     */
    $spec: describe()
  }
}

module.exports = define

==> apeman-api-fs-1.2.1.tgz: package/lib/define.js <==
/**
 * Define api of apeman-api-fs
 * @function define
 * @param {Object} [options] - Optional settings
 * @param {boolean} [options.logging] - Enable logger
 * @param {string} [options.root=process.cwd()] - Root directory path
 * @returns {Object} - API configuration
 */
'use strict'

const co = require('co')
const { mkdirpAsync, existsAsync, statAsync } = require('asfs')
const defaults = require('defaults')
const filemode = require('filemode')
const filelink = require('filelink')
const aglob = require('aglob')
const aslogger = require('aslogger')
const filecopy = require('filecopy')
const apemanApiPath = require('apeman-api-path')
const describe = require('./describe')

let modeString = (mode) => mode.toString(8)

/** @lends define */
function define (options = {}) {
  let logger = aslogger({ disabled: !options.logging })
  let { root } = defaults(options, {
    root: process.cwd()
  })

  let path = apemanApiPath({ root, logging: false })

  return {
    ping (pong = 'pong') {
      return co(function * () {
        logger.trace('pong:', pong)
        return pong
      })
    },
    glob (pattern, options) {
      const s = this
      let args = [ ...arguments ]
      return co(function * () {
        let filenames = yield aglob(...args)
        for (let i = 0; i < filenames.length; i++) {
          let filename = filenames[ i ]
          filenames[ i ] = yield path.resolve(filename)
        }
        return filenames
      })
    },
    exists (filename) {
      const s = this
      return co(function * () {
        let resolved = yield path.resolve(filename)
        return yield existsAsync(resolved)
      })
    },
    stat (filename) {
      const s = this
      return co(function * () {
        let resolved = yield path.resolve(filename)
        return yield statAsync(resolved)
      })
    },
    mkdir (dirname) {
      const s = this
      return co(function * () {
        let result = yield s.mkdirAll([ dirname ])
        return !!result[ 0 ]
      })
    },
    mkdirAll (dirnames) {
      const s = this
      return co(function * () {
        let results = []
        for (let dirname of dirnames) {
          let resolved = yield path.resolve(dirname)
          let exists = yield s.exists(resolved)
          if (exists) {
            continue
          }
          yield mkdirpAsync(resolved)
          logger.debug(`Directory generated: ${dirname}`)
          results.push(dirname)
        }
        return results
      })
    },
    chmod (pattern, mode) {
      const s = this
      return s.chmodAll({ [pattern]: mode })
    },
    chmodAll (permissions) {
      const s = this
      return co(function * () {
        let results = []
        for (let pattern of Object.keys(permissions)) {
          let mode = permissions[ pattern ]
          let filenames = yield s.glob(pattern)
          for (let filename of filenames) {
            let changed = yield filemode(filename, mode, {})
            Object.keys(changed).forEach((filename) => {
              let { from, to } = changed[ filename ]
              logger.debug(`Permission changed: ${filename} ${modeString(from)} to ${modeString(to)}`)
              results.push(filename)
            })
          }
        }
        return results
      })
    },
    copy (src, dest) {
      const s = this
      return s.copyAll({ [src]: dest })
    },
    copyAll (config) {
      const s = this
      return co(function * () {
        let results = []
        for (let src of Object.keys(config)) {
          let dest = config[ src ]
          src = yield path.resolve(src)
          dest = yield path.resolve(dest)
          let copied = yield filecopy(src, dest, {
            mkdirp: true
          })
          let changed = Object.keys(copied).shift()
          if (changed) {
            logger.debug(`File copied: ${src} -> ${dest}`)
            results.push(changed)
          }
        }
        return results
      })
    },
    symlink (src, dest, options = {}) {
      const s = this
      return s.symlinkAll({ [src]: dest }, options)
    },
    symlinkAll (config, options = {}) {
      const s = this
      return co(function * () {
        let results = []
        for (let src of Object.keys(config)) {
          let dest = config[ src ]
          src = yield path.resolve(src)
          let exists = yield s.exists(src)
          if (!exists) {
            console.warn(`Symlink src "${src}" not exists`)
            continue
          }
          dest = yield path.resolve(dest)
          let changed = yield filelink(src, dest, {
            type: 'symlink',
            mkdirp: true,
            force: true
          })
          if (changed) {
            logger.debug(`File linked: ${src} -> ${dest}`)
            results.push({ src, dest })
          }
        }
        return results
      })
    },
    /**
     * A hook which runs before method invoked
     * @param {string} methodName - Name of method to invoke
     * @param {Array} params - Prams for invocation
     * @returns {Promise}
     */
    $before (methodName, params) {
      const s = this
      let { state } = s
      return co(function * () {
        state.invokedAt = new Date()
        logger.info(`${methodName} started...`)
      })
    },
    /**
     * A hook which runs after method invoked
     * @param {string} methodName - Name of method to invoke
     * @param {Array} params - Prams for invocation
     * @param {?*} returns - Return values
     * @returns {Promise}
     */
    $after (methodName, params, returns) {
      const s = this
      let { state } = s
      return co(function * () {
        let took = new Date() - state.invokedAt
        logger.info(`...${methodName} done! (${took}ms)`)
      })
    },
    /**
     * Method specifications
     */
    $spec: describe()
  }
}

module.exports = define
